/*
 * File   : $Source: /usr/local/cvs/opencms/src/org/opencms/file/CmsUser.java,v $
 * Date   : $Date: 2005/10/10 16:11:08 $
 * Version: $Revision: 1.30 $
 *
 * This library is part of OpenCms -
 * the Open Source Content Mananagement System
 *
 * Copyright (c) 2005 Alkacon Software GmbH (http://www.alkacon.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * For further information about Alkacon Software GmbH, please see the
 * company website: http://www.alkacon.com
 *
 * For further information about OpenCms, please see the
 * project website: http://www.opencms.org
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.opencms.file;

import org.opencms.db.CmsDbUtil;
import org.opencms.db.CmsUserSettings;
import org.opencms.main.CmsIllegalArgumentException;
import org.opencms.main.OpenCms;
import org.opencms.security.CmsSecurityException;
import org.opencms.security.I_CmsPrincipal;
import org.opencms.util.CmsStringUtil;
import org.opencms.util.CmsUUID;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 * A user in the OpenCms permission system.<p>
 *
 * A user in OpenCms is uniquely definded by its user named returned by
 * <code>{@link #getName()}</code>.<p>
 * 
 * Basic users in OpenCms are of type <code>{@link #USER_TYPE_SYSTEMUSER}</code>.
 * This means that the user can access the OpenCms Workplace.
 * Moreover, the user must be created by another user with the
 * <code>{@link org.opencms.security.CmsRole#ACCOUNT_MANAGER}</code>.
 * This user type is for "content managers" that actually have write permissions in 
 * at last some parts of the VFS.<p>
 * 
 * Another possible type of users is <code>{@link #USER_TYPE_WEBUSER}</code>.
 * These users do not have access to the OpenCms Workplace. 
 * However, a web user can be created by every user, for example the "Guest" user.
 * The main use case is that web users are used for users of the website that 
 * can generate their own accounts, in a "please register your account..." scenario. 
 * These web user accounts can then be used to 
 * build personalized web sites. A web user is created using 
 * <code>{@link org.opencms.file.CmsObject#addWebUser(String, String, String, String, Map)}</code>.<p>
 *
 * @author Alexander Kandzior 
 * @author Michael Emmerich 
 * 
 * @version $Revision: 1.30 $
 * 
 * @since 6.0.0
 * 
 * @see CmsGroup 
 */
public class CmsUser implements I_CmsPrincipal, Cloneable {

    /** A user-type system user. */
    public static final int USER_TYPE_SYSTEMUSER = 0;

    /** A user-type web user. */
    public static final int USER_TYPE_WEBUSER = 2;

    /** A storage for additional user information. */
    private Map m_additionalInfo;

    /** The address of this user. */
    private String m_address;

    /** The description of the user. */
    private String m_description;

    /**  The email of the user. */
    private String m_email;

    /** The first name of this user. */
    private String m_firstname;

    /** The flags of the user. */
    private int m_flags;

    /** The id of this user. */
    private CmsUUID m_id;

    /** Boolean flag whether the last-login timestamp of this user was modified. */
    private boolean m_isTouched;

    /** The last login date of this user. */
    private long m_lastlogin;

    /** The last name of this user. */
    private String m_lastname;

    /** The unique user name of this user. */
    private String m_name;

    /** The password of this user. */
    private String m_password;

    /**
     * Defines if the user is a webuser or a systemuser.<p>
     * 
     * USER_TYPE_SYSTEMUSER for systemuser (incl. guest).
     * USER_TYPE_WEBUSER for webuser.
     */
    private int m_type;

    /**
     * Creates a new empty CmsUser object.<p>
     *
     * Only intented to be used with the org.opencms.workplace.tools.users.CmsEditUserDialog.<p>
     */
    public CmsUser() {

        m_name = "";
        m_description = "";
        m_additionalInfo = new HashMap();
        m_address = "";
        m_email = "";
        m_firstname = "";
        m_flags = I_CmsPrincipal.FLAG_ENABLED;
        m_lastlogin = CmsDbUtil.UNKNOWN_ID;
        m_lastname = "";
        m_password = "";
        m_type = CmsDbUtil.UNKNOWN_ID;
        m_isTouched = false;
    }

    /**
     * Creates a new Cms user object.<p>
     *
     * @param id the id of the new user
     * @param name the name of the new user
     * @param description the description of the new user
     */
    public CmsUser(CmsUUID id, String name, String description) {

        m_id = id;
        m_name = name;
        m_description = description;
        m_additionalInfo = null;
        m_address = "";
        m_email = "";
        m_firstname = "";
        m_flags = I_CmsPrincipal.FLAG_ENABLED;
        m_lastlogin = CmsDbUtil.UNKNOWN_ID;
        m_lastname = "";
        m_password = "";
        m_type = CmsDbUtil.UNKNOWN_ID;
        m_isTouched = false;
    }

    /**
     * Creates a new CmsUser object.<p>
     * @param id the id of the new user
     * @param name the name of the new user
     * @param password the password of the user
     * @param description the description of the new user
     * @param firstname the first name
     * @param lastname the last name
     * @param email the email address
     * @param lastlogin time stamp 
     * @param flags flags
     * @param additionalInfo user related information
     * @param address the address
     * @param type the type of this user
     */
    public CmsUser(
        CmsUUID id,
        String name,
        String password,
        String description,
        String firstname,
        String lastname,
        String email,
        long lastlogin,
        int flags,
        Map additionalInfo,
        String address,
        int type) {

        m_id = id;
        m_name = name;
        m_password = password;
        m_description = description;
        m_firstname = firstname;
        m_lastname = lastname;
        m_email = email;
        m_lastlogin = lastlogin;
        m_flags = flags;
        m_additionalInfo = additionalInfo;
        m_address = address;
        m_type = type;
    }

    /**
     * Validates an email address.<p>
     * 
     * That means, the parameter should only be composed by digits and standard english letters, points, underscores and exact one "At" symbol.<p>
     * 
     * @param email the email to validate
     */
    public static void checkEmail(String email) {

        if (!CmsStringUtil.validateRegex(
            email,
            "^([a-zA-Z0-9_\\.\\-])+\\@(([a-zA-Z0-9\\-])+\\.)+([a-zA-Z0-9]{2,4})+$",
            false)) {
            throw new CmsIllegalArgumentException(Messages.get().container(Messages.ERR_EMAIL_VALIDATION_1, email));
        }
    }

    /**
     * Validates a login.<p>
     * 
     * That means, the parameter should only be composed by digits and standard english letters, points, minus and underscores.<p>
     * 
     * @param login the login to validate
     */
    public static void checkLogin(String login) {

        if (!CmsStringUtil.validateRegex(login, "[\\w\\.\\-~_]*", false)) {
            throw new CmsIllegalArgumentException(Messages.get().container(Messages.ERR_LOGIN_VALIDATION_1, login));
        }
    }

    /**
     * Validates a zip code.<p>
     * 
     * That means, the parameter should only be composed by digits and standard english letters.<p>
     * 
     * @param zipcode the zipcode to validate
     */
    public static void checkZipCode(String zipcode) {

        if (!CmsStringUtil.validateRegex(zipcode, "[\\w]*", true)) {
            throw new CmsIllegalArgumentException(Messages.get().container(Messages.ERR_ZIPCODE_VALIDATION_1, zipcode));
        }
    }

    /**
     * Returns the "full" name the given user in the format "{firstname} {lastname} ({username})",
     * or the empty String "" if the user is null.<p>
     * 
     * @param user the user to get the full name from
     * @return the "full" name the user
     * @see #getFullName() 
     */
    public static String getFullName(CmsUser user) {

        if (user == null) {
            return "";
        } else {
            return user.getFullName();
        }
    }

    /**
     * Returns <code>true</code> if the provided user type indicates a system user type.<p>
     * 
     * @param type the user type
     * @return true if the provided user type indicates a system user type
     */
    public static boolean isSystemUser(int type) {

        return (type & 1) > 0;
    }

    /**
     * Returns <code>true</code> if the provided user type indicates a web user type.<p>
     * 
     * @param type the user type
     * @return true if the provided user type indicates a web user type
     */
    public static boolean isWebUser(int type) {

        return (type & 2) > 0;
    }

    /**
     * Returns a clone of this Objects instance.<p>
     * 
     * @return a clone of this instance
     */
    public Object clone() {

        CmsUser user = new CmsUser(
            m_id,
            new String(m_name),
            new String(m_password),
            new String(m_description),
            new String(m_firstname),
            new String(m_lastname),
            new String(m_email),
            m_lastlogin,
            m_flags,
            getAdditionalInfo(),
            new String(m_address),
            m_type);
        return user;
    }

    /**
     * Delete additional information about the user. <p>
     * Additional infos are for example emailadress, adress or surname...<BR/><BR/>
     *
     * @param key The key of the additional information to delete
     */
    public void deleteAdditionalInfo(String key) {

        m_additionalInfo.remove(key);
    }

    /**
     * @see java.lang.Object#equals(java.lang.Object)
     */
    public boolean equals(Object obj) {

        if (obj == this) {
            return true;
        }
        if (obj instanceof CmsUser) {
            return (((CmsUser)obj).m_id.equals(m_id));
        }
        return false;
    }

    /**
     * Returns the complete Hashtable with additional information about the user.<p>
     * Additional infos are for example emailadress, adress or surname...
     *
     * The additional infos must be requested via the CmsObject.
     *
     * @return additional information about the user
     *
     */
    public Map getAdditionalInfo() {

        return m_additionalInfo;
    }

    /**
     * Returns additional information about the user which are usually set 
     * in the users preferences.<p>
     *
     * @param key the key to the additional information
     * @return additional information Object about the user, if the additional info
     * does not exists, it returns <code>null</code>
     */
    public Object getAdditionalInfo(String key) {

        return m_additionalInfo.get(key);
    }

    /**
     * Gets the address.<p>
     *
     * @return the USER_ADDRESS, or null
     */
    public String getAddress() {

        return m_address;
    }

    /**
     * Returns the city of the address of the user.<p>
     * 
     * @return the city
     */
    public String getCity() {

        return (String)getAdditionalInfo(CmsUserSettings.ADDITIONAL_INFO_TOWN);
    }

    /**
     * Returns the country of the address of the user.<p>
     * 
     * @return the country
     */
    public String getCountry() {

        return (String)getAdditionalInfo(CmsUserSettings.ADDITIONAL_INFO_COUNTRY);
    }

    /**
     * Gets the description of this user.<p>
     *
     * @return the description of this user
     */
    public String getDescription() {

        return m_description;
    }

    /**
     * Decides if this user is disabled.<p>
     *
     * @return USER_FLAGS == FLAG_DISABLED
     */
    public boolean getDisabled() {

        return (getFlags() & I_CmsPrincipal.FLAG_DISABLED) == I_CmsPrincipal.FLAG_DISABLED;
    }

    /**
     * Returns the email address of this user.<p>
     *
     * @return the email address of this user
     */
    public String getEmail() {

        return m_email;
    }

    /**
     * Returns the firstname of this user.<p>
     *
     * @return the firstname of this user
     */
    public String getFirstname() {

        return m_firstname;
    }

    /**
     * Returns the flags of this user.<p>
     *
     * @return the flags of this user
     */
    public int getFlags() {

        return m_flags;
    }

    /**
     * Returns the "full" name this user in the format "{firstname} {lastname} ({username})".<p>
     * 
     * @return the "full" name this user 
     */
    public String getFullName() {

        StringBuffer buf = new StringBuffer();
        String first = getFirstname();
        if ((null != first) && !"".equals(first.trim())) {
            buf.append(first);
            buf.append(" ");
        }
        String last = getLastname();
        if ((null != last) && !"".equals(last.trim())) {
            buf.append(getLastname());
            buf.append(" ");
        }
        buf.append("(");
        buf.append(getName());
        buf.append(")");
        return buf.toString();
    }

    /**
     * Returns the id of this user.<p>
     *
     * @return the id of this user
     */
    public CmsUUID getId() {

        return m_id;
    }

    /**
     * Returns the time of the last login of this user.<p>
     *
     * @return the time of the last login of this user
     */
    public long getLastlogin() {

        return m_lastlogin;
    }

    /**
     * Returns the lastname of this user.<p>
     *
     * @return the lastname of this user
     */
    public String getLastname() {

        return m_lastname;
    }

    /**
     * Gets the (login) name of this user.<p>
     *
     * @return the (login) name of this user
     */
    public String getName() {

        return m_name;
    }

    /**
     * Returns the encrypted user password.<p>
     *
     * @return the encrypted user password
     */
    public String getPassword() {

        return m_password;
    }

    /**
     * Returns the type of the user.<p>
     * 
     * Possible options are
     * <code>{@link #USER_TYPE_SYSTEMUSER}</code> for a system user (incliding the "Guest" user),
     * or <code>{@link #USER_TYPE_WEBUSER}</code> for a webuser.<p>
     *
     * @return the type
     */
    public int getType() {

        return m_type;
    }

    /**
     * Returns the zip code of the address of the user.<p>
     * 
     * @return the zip code 
     */
    public String getZipcode() {

        return (String)getAdditionalInfo(CmsUserSettings.ADDITIONAL_INFO_ZIPCODE);
    }

    /**
     * @see java.lang.Object#hashCode()
     */
    public int hashCode() {

        if (m_id != null) {
            return m_id.hashCode();
        }
        return CmsUUID.getNullUUID().hashCode();
    }

    /**
     * Returns the enabled flag.<p>
     * 
     * It should replace the <code>{@link #getDisabled()}</code> method.<p> 
     * 
     * @return the enabled flag
     */
    public boolean isEnabled() {

        return (getFlags() & I_CmsPrincipal.FLAG_DISABLED) == 0;
    }

    /**
     * Returns <code>true</code> if this user is the default guest user.<p>
     * 
     * @return true if this user is the default guest user
     */
    public boolean isGuestUser() {

        return OpenCms.getDefaultUsers().getUserGuest().equals(getName());
    }

    /**
     * Returns <code>true</code> if this user is a system user.<p>
     * 
     * @return true if this user is a system user
     */
    public boolean isSystemUser() {

        return isSystemUser(m_type);
    }

    /**
     * Returns true if this user was touched, e.g. the last-login timestamp was changed.<p>
     * 
     * @return boolean true if this resource was touched
     */
    public boolean isTouched() {

        return m_isTouched;
    }

    /**
     * Returns <code>true</code> if this user is a web user.<p>
     * 
     * @return true if this user is a web user
     */
    public boolean isWebUser() {

        return isWebUser(m_type);
    }

    /**
     * Sets the  complete Hashtable with additional information about the user. <p>
     * Additional infos are for example emailadress, adress or surname...<BR/><BR/>
     *
     * This method has package-visibility for security-reasons.
     * It is required to because of the use of two seprate databases for user data and
     * additional user data.
     * 
     * @param additionalInfo user-related additional information
     *
     */
    public void setAdditionalInfo(Map additionalInfo) {

        m_additionalInfo = additionalInfo;
    }

    /**
     * Sets additional information about the user. <p>
     * Additional infos are for example emailadress, adress or surname...<BR/><BR/>
     *
     *
     * @param key The key to the additional information
     * @param obj The additinoal information value
     *
     */
    public void setAdditionalInfo(String key, Object obj) {

        m_additionalInfo.put(key, obj);
    }

    /**
     * Sets the address.<p>
     *
     * @param value The user adress
     */
    public void setAddress(String value) {

        m_address = value;
    }

    /**
     * Sets the city of the address of the user.<p>
     * 
     * @param city the city
     */
    public void setCity(String city) {

        setAdditionalInfo(CmsUserSettings.ADDITIONAL_INFO_TOWN, city);
    }

    /**
     * Sets the country of the address of the user.<p>
     * 
     * @param country the country
     */
    public void setCountry(String country) {

        setAdditionalInfo(CmsUserSettings.ADDITIONAL_INFO_COUNTRY, country);
    }

    /**
     * Sets the description of this user.<p>
     *
     * @param value the description of this user
     */
    public void setDescription(String value) {

        m_description = value;
    }

    /**
     * Disables the user flags by setting them to FLAG_DISABLED.<p>
     */
    public void setDisabled() {

        if (isEnabled()) {
            setFlags(getFlags() ^ I_CmsPrincipal.FLAG_DISABLED);
        }
    }

    /**
     * Sets the email.<p>
     *
     * @param value The new email adress
     */
    public void setEmail(String value) {

        checkEmail(value);
        m_email = value;
    }

    /**
     * Enables the user flags by setting them to FLAG_ENABLED.<p>
     */
    public void setEnabled() {

        if (getDisabled()) {
            setFlags(getFlags() ^ I_CmsPrincipal.FLAG_DISABLED);
        }
    }

    /**
     * Sets the enabled flag.<p>
     * 
     * It should replace the <code>{@link #setDisabled()}</code> and 
     * the <code>{@link #setEnabled()}</code> methods.<p> 
     * 
     * @param enabled the enabled flag
     */
    public void setEnabled(boolean enabled) {

        if (enabled) {
            setEnabled();
        } else {
            setDisabled();
        }
    }

    /**
     * Sets the firstname.<p>
     *
     * @param firstname the USER_FIRSTNAME
     */
    public void setFirstname(String firstname) {

        if (CmsStringUtil.isEmptyOrWhitespaceOnly(firstname)) {
            throw new CmsIllegalArgumentException(Messages.get().container(Messages.ERR_FIRSTNAME_EMPTY_0));
        }
        m_firstname = firstname;
    }

    /**
     * Sets the flags.<p>
     *
     * @param value The new user flags
     */
    public void setFlags(int value) {

        m_flags = value;
    }

    /**
     * Sets the lastlogin.<p>
     *
     * @param value The new user section
     */
    public void setLastlogin(long value) {

        m_isTouched = true;
        m_lastlogin = value;
    }

    /**
     * Gets the lastname.<p>
     *
     * @param lastname the last name of the user
     */
    public void setLastname(String lastname) {

        if (CmsStringUtil.isEmptyOrWhitespaceOnly(lastname)) {
            throw new CmsIllegalArgumentException(Messages.get().container(Messages.ERR_LASTNAME_EMPTY_0));
        }
        m_lastname = lastname;
    }

    /**
     * Sets the name (login).<p>
     *
     * @param name the name (login) to set
     */
    public void setName(String name) {

        checkLogin(name);
        m_name = name;
    }

    /**
     * Sets the password.<p>
     *
     * @param value The new password
     */
    public void setPassword(String value) {

        try {
            OpenCms.getPasswordHandler().validatePassword(value);
        } catch (CmsSecurityException e) {
            throw new CmsIllegalArgumentException(e.getMessageContainer());
        }
        m_password = value;
    }

    /**
     * Sets the zip code of the address of the user.<p>
     * 
     * @param zipcode the zip code 
     */
    public void setZipcode(String zipcode) {

        checkZipCode(zipcode);
        zipcode = zipcode.toUpperCase();
        setAdditionalInfo(CmsUserSettings.ADDITIONAL_INFO_ZIPCODE, zipcode);
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString() {

        StringBuffer result = new StringBuffer();
        result.append("[User]");
        result.append(" name:");
        result.append(m_name);
        result.append(" id:");
        result.append(m_id);
        result.append(" flags:");
        result.append(getFlags());
        result.append(" type:");
        result.append(getType());
        result.append(" description:");
        result.append(m_description);
        return result.toString();
    }

    /**
     * Sets the "touched" status of this user to "true".<p>
     */
    public void touch() {

        m_isTouched = true;
    }

    /**
     * Sets the typ of this user.<p>
     *
     * @param value the type of this user
     */
    /*package*/void setType(int value) {

        m_type = value;
    }

    /**
     * Filters out all users with flags greater than <code>{@link I_CmsPrincipal#FLAG_CORE_LIMIT}</code>.<p>
     * 
     * @param users the list of <code>{@link CmsUser}</code>
     * 
     * @return the same filtered list
     */
    public static List filterCore(List users) {
    
        Iterator it = users.iterator();
        while (it.hasNext()) {
            CmsUser user = (CmsUser)it.next();
            if (user.getFlags() > I_CmsPrincipal.FLAG_CORE_LIMIT) {
                it.remove();
            }
        }
        return users;
    }

    /**
     * Filters out all users that does not have the given flag set.<p>
     * 
     * @param users the list of <code>{@link CmsUser}</code>
     * @param flag the flag for filtering
     * 
     * @return the same filtered list
     */
    public static List filterFlag(List users, int flag) {
    
        Iterator it = users.iterator();
        while (it.hasNext()) {
            CmsUser user = (CmsUser)it.next();
            if ((user.getFlags() & flag) != flag) {
                it.remove();
            }
        }
        return users;
    }

    /**
     * Filters out all users that does not have the given flag set,
     * but leaving all users with flags less than <code>{@link I_CmsPrincipal#FLAG_CORE_LIMIT}</code>.<p>
     * 
     * @param users the list of <code>{@link CmsUser}</code>
     * @param flag the flag for filtering
     * 
     * @return the same filtered list
     */
    public static List filterCoreFlag(List users, int flag) {
    
        Iterator it = users.iterator();
        while (it.hasNext()) {
            CmsUser user = (CmsUser)it.next();
            if (user.getFlags() > I_CmsPrincipal.FLAG_CORE_LIMIT && (user.getFlags() & flag) != flag) {
                it.remove();
            }
        }
        return users;
    }
}