/*
 * File   : $Source: /usr/local/cvs/opencms/src/org/opencms/widgets/CmsHtmlAreaWidget.java,v $
 * Date   : $Date: 2005/06/28 17:27:52 $
 * Version: $Revision: 1.12 $
 *
 * This library is part of OpenCms -
 * the Open Source Content Mananagement System
 *
 * Copyright (c) 2005 Alkacon Software GmbH (http://www.alkacon.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * For further information about Alkacon Software GmbH, please see the
 * company website: http://www.alkacon.com
 *
 * For further information about OpenCms, please see the
 * project website: http://www.opencms.org
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.opencms.widgets;

import org.opencms.file.CmsObject;
import org.opencms.i18n.CmsEncoder;
import org.opencms.main.OpenCms;
import org.opencms.workplace.CmsWorkplace;

import java.util.Iterator;
import java.util.Locale;
import java.util.Map;

/**
 * Provides a widget that creates a rich input field using the "HtmlArea" component, for use on a widget dialog.<p>
 *
 * @author Alexander Kandzior 
 * 
 * @version $Revision: 1.12 $ 
 * 
 * @since 6.0.0 
 */
public class CmsHtmlAreaWidget extends A_CmsWidget {
    
    /** VFS path to the available HtmlArea Locales. */
    public static final String HTMLAREA_LOCALES_VFS = CmsWorkplace.VFS_PATH_WORKPLACE + "resources/editors/htmlarea/lang/";

    /**
     * Creates a new html area editor widget.<p>
     */
    public CmsHtmlAreaWidget() {

        // empty constructor is required for class registration
        this("");
    }

    /**
     * Creates a new html area editor widget with the given configuration.<p>
     * 
     * @param configuration the configuration to use
     */
    public CmsHtmlAreaWidget(String configuration) {

        super(configuration);
    }
    
    /**
     * Returns the Locale for which HtmlArea has localization data available.<p>
     * 
     * @param cms the initialized cmsobject
     * @param wantedLocale the preferred Locale
     * @return the Locale for which HtmlArea has localization data available
     */
    public static Locale getHtmlAreaLocale(CmsObject cms, Locale wantedLocale) {
        
        // check the Locale to use for editor localization
        if (!cms.existsResource(HTMLAREA_LOCALES_VFS + wantedLocale.toString() + ".js")) {
            boolean foundLocale = false;
            Iterator i = OpenCms.getLocaleManager().getDefaultLocales().iterator();
            // check the available default Locales
            while (i.hasNext()) {
                wantedLocale = (Locale)i.next();
                if (cms.existsResource(HTMLAREA_LOCALES_VFS + wantedLocale.toString() + ".js")) {
                    // found a valid Locale
                    foundLocale = true;
                    break;
                }
            }
            if (! foundLocale) {
                // did not find any matching Locale, fall back to english
                wantedLocale = Locale.ENGLISH;
            }
        }
        return wantedLocale;
    }

    /**
     * @see org.opencms.widgets.I_CmsWidget#getDialogIncludes(org.opencms.file.CmsObject, org.opencms.widgets.I_CmsWidgetDialog)
     */
    public String getDialogIncludes(CmsObject cms, I_CmsWidgetDialog widgetDialog) {
        
        StringBuffer result = new StringBuffer(16);
        result.append("<script type=\"text/javascript\">\n<!--\n");
        result.append("\t_editor_url = \"");
        result.append(CmsWorkplace.getSkinUri());
        result.append("editors/htmlarea/\";\n");
        result.append("\t_editor_lang = \"");
        result.append(getHtmlAreaLocale(cms, widgetDialog.getLocale()));
        result.append("\";\n");
        result.append("//-->\n</script>\n");
        result.append(getJSIncludeFile(CmsWorkplace.getSkinUri() + "editors/htmlarea/htmlarea.js"));
        result.append("\n");
        result.append(getJSIncludeFile(CmsWorkplace.getSkinUri() + "components/widgets/htmlarea.js"));
        return result.toString();
    }

    /**
     * @see org.opencms.widgets.I_CmsWidget#getDialogInitCall(org.opencms.file.CmsObject, org.opencms.widgets.I_CmsWidgetDialog)
     */
    public String getDialogInitCall(CmsObject cms, I_CmsWidgetDialog widgetDialog) {

        // the timeout setting prevents IE from jumping to the last html area widget
        return "\tinitHtmlArea();\n";
    }

    /**
     * @see org.opencms.widgets.I_CmsWidget#getDialogInitMethod(org.opencms.file.CmsObject, org.opencms.widgets.I_CmsWidgetDialog)
     */
    public String getDialogInitMethod(CmsObject cms, I_CmsWidgetDialog widgetDialog) {

        StringBuffer result = new StringBuffer(8);
        result.append("function initHtmlArea() {\n");
        result.append("\tinitHtmlAreas();\n");
        result.append("}\n");
        return result.toString();
    }

    /**
     * @see org.opencms.widgets.I_CmsWidget#getDialogWidget(org.opencms.file.CmsObject, org.opencms.widgets.I_CmsWidgetDialog, org.opencms.widgets.I_CmsWidgetParameter)
     */
    public String getDialogWidget(CmsObject cms, I_CmsWidgetDialog widgetDialog, I_CmsWidgetParameter param) {

        String id = param.getId();
        StringBuffer result = new StringBuffer(128);
      
        result.append("<td class=\"xmlTd\">");
        result.append("<textarea class=\"xmlInput maxwidth\" name=\"");
        result.append(id);
        result.append("\" id=\"");
        result.append(id);
        result.append("\" rows=\"15\" wrap=\"virtual\">");
        result.append(CmsEncoder.escapeXml(param.getStringValue(cms)));
        result.append("</textarea>");
        result.append("</td>");

        return result.toString();
    }

    /**
     * @see org.opencms.widgets.I_CmsWidget#newInstance()
     */
    public I_CmsWidget newInstance() {

        return new CmsHtmlAreaWidget(getConfiguration());
    }

    /**
     * @see org.opencms.widgets.I_CmsWidget#setEditorValue(org.opencms.file.CmsObject, java.util.Map, org.opencms.widgets.I_CmsWidgetDialog, org.opencms.widgets.I_CmsWidgetParameter)
     */
    public void setEditorValue(
        CmsObject cms,
        Map formParameters,
        I_CmsWidgetDialog widgetDialog,
        I_CmsWidgetParameter param) {

        String[] values = (String[])formParameters.get(param.getId());
        if ((values != null) && (values.length > 0)) {
            String val = CmsEncoder.decode(values[0], CmsEncoder.ENCODING_UTF_8);
            param.setStringValue(cms, val);
        }
    }
}